@@ -33,37 +33,55 @@ import datetime
 import random
 import logging
 import os
+import json
 
 RT_TIMEOUT = 300  # in seconds
+TCP_TIMEOUT = 20  # in seconds
+PENALTY_TIME = 0*60  # in seconds
 
 
 ###############################################################################
 class DataRouter(tornado.web.Application):
     # -------------------------------------------------------------------------
-    def __init__(self, timeout=60, logger=None, *args, **kwds):
+    def __init__(self, logger=None, *args, **kwds):
         super().__init__(*args, **kwds)
         self.logger = logger or logging.getLogger(__name__)
-        self.timeout = datetime.timedelta(seconds=timeout)
 
         config = load_system_configuration()
         self.mc_servers = [config.get("memcache", "url")]
         self.dataservers = []
+        self.penalty_box = {}
 
     # -------------------------------------------------------------------------
-    def add_dataserver(self, server_id):
-        if server_id not in self.dataservers:
-            self.dataservers.append(server_id)
-            self.logger.info("{0!s} registered "
-                "as available server".format(server_id))
-            return True
-        return False
+    def add_dataserver(self, addr, port, hostname):
+        server_id = (addr, port, hostname)
+
+        # --- don't register the same server multiple times
+        if server_id in self.dataservers:
+            return False
+
+        # --- if server was put in the penalty box, check that penalty time has
+        #     expired (this to avoid adding the same unresponsive server over
+        #     and over)
+        if addr in self.penalty_box:
+            dt = Date.now() - self.penalty_box[addr]
+            if dt.seconds > PENALTY_TIME:
+                del self.penalty_box[addr]
+            else:
+                self.logger.info(
+                    f"Registration of {addr!s} ({hostname:s}) refused")
+                return False
+
+        self.dataservers.append(server_id)
+        self.logger.info(f"{server_id!s} registered as available server")
+        return True
 
     # -------------------------------------------------------------------------
     def drop_dataserver(self, server_id, msg):
-        self.logger.info("dropping "
-            "dataserver {0!s}: {1:s}".format(server_id, msg))
+        self.logger.info(f"dropping dataserver {server_id!s}: {msg:s}")
         try:
             self.dataservers.remove(server_id)
+            self.penalty_box[server_id[0]] = Date.now()
         except ValueError:
             pass
 
@@ -86,10 +104,7 @@ class DataRouter(tornado.web.Application):
             win32file._setmaxstdio(2048)
             assert win32file._getmaxstdio() == 2048
 
-        # --- start HTTP server with an idle_connection_timeout so that idle
-        #     connections don't stick around for more than 90 seconds
-        http_server = tornado.httpserver.HTTPServer(
-            self, idle_connection_timeout=90)
+        http_server = tornado.httpserver.HTTPServer(self)
         http_server.listen(port)
 
         # --- start the memcache-client
@@ -111,21 +126,27 @@ class DataRouter(tornado.web.Application):
         self.logger.info("shutting down router")
 
     # -------------------------------------------------------------------------
-    async def with_timeout(self, future):
-        return await tornado.gen.with_timeout(self.timeout, future)
+    async def with_timeout(self, future, timeout):
+        return await tornado.gen.with_timeout(
+            datetime.timedelta(seconds=timeout), future)
 
     # -------------------------------------------------------------------------
     async def fetch(self, req, addr, port):
         client = tornado.tcpclient.TCPClient()
-        stream = await self.with_timeout(client.connect(addr, port))
+        stream = await self.with_timeout(client.connect(addr, port), 1)
 
         try:
-            await self.with_timeout(stream.write(encode_message(req)))
-            response = await self.with_timeout(stream.read_until(b"\n"))
+            req = encode_message(req)
+            await self.with_timeout(stream.write(req), TCP_TIMEOUT)
+            response = await self.with_timeout(
+                stream.read_until(b"\n"), TCP_TIMEOUT)
         finally:
             try:
                 stream.close()
             except tornado.iostream.StreamClosedError:
+                # --- this will ignore StreamClosedError raised when closing
+                #     the connection, but StreamClosedErrors raised before will
+                #     still be propagated
                 pass
 
         return response
@@ -146,15 +167,16 @@ class DataRouter(tornado.web.Application):
         if response is None:
             # --- no valid data stored in cache, send request to datafeed
             #     router.
-            #     NB: we first lock the cache with a timeout and then we only
-            #         set the cache if the response is valid.
+            #     NB: we first lock the cache with a timeout (longer than the
+            #         one used for requests) and then we  only set the cache if
+            #         the response is valid.
             await self.cache.set(
-                req_uid, "CacheLocked", self.timeout.seconds, noreply=True)
+                req_uid, "CacheLocked", 10*TCP_TIMEOUT, noreply=True)
 
             while True:
                 # --- select dataserver or return error if none is available
                 try:
-                    addr, port = self.select_dataserver()
+                    addr, port, hostname = self.select_dataserver()
                 except NoDataserverAvailable:
                     resp = {
                         "type": "NoDataserverAvailable",
@@ -168,7 +190,7 @@ class DataRouter(tornado.web.Application):
                         tornado.iostream.StreamClosedError) as err:
                     # --- connection unresponsive: drop dataserver from list
                     #     and process the request again
-                    self.drop_dataserver((addr, port), msg=str(err))
+                    self.drop_dataserver((addr, port, hostname), msg=str(err))
                     continue
                 else:
                     break
@@ -199,13 +221,18 @@ class RegistrationHandler(tornado.web.RequestHandler):
 
         addr = self.get_argument("address")
         port = self.get_argument("port")
+        hostname = self.get_argument("hostname", "unknown")
 
-        added = self.application.add_dataserver((addr, port))
+        added = self.application.add_dataserver(addr, port, hostname)
         if added:
             self.set_status(201)
         else:
             self.set_status(205)
 
+    # -------------------------------------------------------------------------
+    async def get(self):
+        self.write(json.dumps(self.application.dataservers))
+
 
 ###############################################################################
 class BbgBDPHandler(tornado.web.RequestHandler):
