@@ -97,15 +97,15 @@ def process_response(resp):
 
 
 # -----------------------------------------------------------------------------
-def get_address_and_port(sockets, dns_suffix):
+def get_address_port_hostname(sockets, dns_suffix):
     port = sockets[0].getsockname()[1]
     hostname = socket.gethostname()
     for addr in socket.gethostbyname_ex(hostname)[2]:
         info = socket.gethostbyaddr(addr)
         if info[0].endswith(dns_suffix):
-            return port, addr
+            return port, addr, hostname
     raise RuntimeError(
-        "cannot detect IP Address with DNS suffix {0!s}".format(dns_suffix))
+        f"cannot detect IP Address with DNS suffix {dns_suffix!s}")
 
 
 ###############################################################################
@@ -137,28 +137,30 @@ class DataServer(tornado.tcpserver.TCPServer):
         sockets = tornado.netutil.bind_sockets(0, address="")
         self.add_sockets(sockets)
 
-        # --- get the port and the IP address
-        port, addr = get_address_and_port(sockets, dns_suffix)
+        # --- get the port, the IP address and the host name
+        port, addr, hostname = get_address_port_hostname(sockets, dns_suffix)
 
         # --- create the request object used to notify the router that this
         #     server is available
         self.subreq = tornado.httpclient.HTTPRequest(
-            "http://{0!s}:{1!s}/register/".format(router_addr, router_port),
+            f"http://{router_addr!s}:{router_port!s}/register/",
             method="PUT",
             headers={"Content-Type": "application/x-www-form-urlencoded"},
-            body=urllib.parse.urlencode({"address": addr, "port": port}),
+            body=urllib.parse.urlencode({
+                "address": addr,
+                "port": port,
+                "hostname": hostname}),
             request_timeout=self.tcp_timeout.seconds
         )
 
         ioloop = tornado.ioloop.IOLoop.current()
-        ioloop.add_callback(self.subscribe_to_router)
 
+        ioloop.add_callback(self.subscribe_to_router)
         if tornado.version < "5.0.0":
-            tornado.ioloop.PeriodicCallback(
-                self.subscribe_to_router, REREGISTER, ioloop).start()
+            args = (self.subscribe_to_router, REREGISTER, ioloop)
         else:
-            tornado.ioloop.PeriodicCallback(
-                self.subscribe_to_router, REREGISTER).start()
+            args = (self.subscribe_to_router, REREGISTER)
+        tornado.ioloop.PeriodicCallback(*args).start()
 
         if stop_at is not None:
             seconds = (stop_at - Date.now()).seconds
@@ -168,40 +170,49 @@ class DataServer(tornado.tcpserver.TCPServer):
             ioloop.start()
         except KeyboardInterrupt:
             self.stop_server()
+        finally:
+            # --- make sure all pending tasks have been terminated
+            while ioloop.asyncio_loop.is_running():
+                self.logger.info("ioloop still running, wait for it to stop")
+                time.sleep(2)
+            self.logger.info("server has been stopped")
 
     # -------------------------------------------------------------------------
-    def stop_server(self):
-        self.logger.info("shutting down server...")
-        # --- first stop listening for new connections
-        super().stop()
-        # --- then stop the threadpool (this will not return until all the
-        #     pending futures are done )
-        self.tpool.shutdown(wait=True)
-        # --- finally stop the tornado event loop
-        ioloop = tornado.ioloop.IOLoop.current()
-        ioloop.stop()
-        self.logger.info("server has been stopped")
+    def thrd_async(self, func, *args):
+        ioloop = tornado.ioloop.IOLoop.instance()
+        return ioloop.run_in_executor(self.tpool, func, *args)
 
     # -------------------------------------------------------------------------
     async def with_timeout(self, future):
         return await tornado.gen.with_timeout(self.tcp_timeout, future)
 
+    # -------------------------------------------------------------------------
+    def stop_server(self):
+        self.logger.info("shutting down server...")
+        # --- stop listening for new connections
+        super().stop()
+        # --- then stop the threadpool (this will not return until all the
+        #     pending futures are done)
+        self.tpool.shutdown(wait=True)
+        # --- finally stop the ioloop itself
+        tornado.ioloop.IOLoop.current().stop()
+
     # -------------------------------------------------------------------------
     async def handle_stream(self, stream, address):
         # --- use time.time to monitor time needed to process request
         t_start = time.time()
 
-        self.logger.debug("connection received from {0!s}".format(address))
+        self.logger.debug(f"connection received from {address!s}")
 
         try:
             request = await self.with_timeout(stream.read_until(b"\n"))
         except CONN_ERRORS as err:
             self.logger.error(
-                "couldn't read request from {0!s}: {1!s}".format(address, err))
+                f"couldn't read request from {address!s}: {err!s}")
             return
 
         request = decode_message(request)
-        self.logger.debug("processing request {0!s}".format(request))
+        self.logger.debug(f"processing request {request!s}")
 
         request_type = request.pop("type")
 
@@ -213,22 +224,19 @@ class DataServer(tornado.tcpserver.TCPServer):
         #     geometrically increasing timeouts
         timeout = self.blp_timeout
         for _ in range(5):
-            self.logger.debug("timeout set to {0:d}".format(timeout))
             try:
                 if request_type == "BDP":
                     # --- pull a bbg-bdp client from the clients pool
                     clt = next(self.bdp_clients)
-                    resp = await self.tpool.submit(
+                    resp = await self.thrd_async(
                                 bdp_request, clt, timeout, request)
                 else:
-                    resp = await self.tpool.submit(
-                                bdh_request, timeout, request)
+                    resp = await self.thrd_async(bdh_request, timeout, request)
                 break
             except TimeoutError:
                 timeout *= 2
-                self.logger.info(
-                    "request {0!s} "
-                    "timeout increased to {1:f}ms".format(request, timeout))
+                self.logger.debug(
+                    f"request {request!s} timeout increased to {timeout:f}ms")
             except DatafeedFatal as err:
                 # --- unrecoverable bloomberg error: don't send a reply so that
                 #     the datafeed router knows this server is currently unable
@@ -242,7 +250,7 @@ class DataServer(tornado.tcpserver.TCPServer):
                 "type": "TimeoutError",
                 "payload": (
                     "request timed out after 5 attempts with a final "
-                    "timeout of {0!s} milliseconds").format(timeout),
+                    f"timeout of {timeout!s} milliseconds"),
             }
             # --- try re-initializing clients
             for clt in self.bdp_clients:
@@ -256,12 +264,12 @@ class DataServer(tornado.tcpserver.TCPServer):
             await self.with_timeout(stream.write(resp))
         except CONN_ERRORS as err:
             self.logger.error(
-                "couldn't send resp to {0!s}: {1!s}".format(address, err))
+                f"couldn't send resp to {address!s}: {err!s}")
             return
 
         time_total = (time.time() - t_start)*1000.0
         self.logger.info(
-            "request {0!s} processed in {1:f}ms".format(request, time_total))
+            f"request {request!s} processed in {time_total:f}ms")
 
     # -------------------------------------------------------------------------
     async def subscribe_to_router(self):
@@ -274,21 +282,20 @@ class DataServer(tornado.tcpserver.TCPServer):
             return
 
         self.logger.debug(
-            "trying to register "
-            "with datafeed router on {0!s}".format(self.subreq.url))
+            f"trying to register with datafeed router on {self.subreq.url!s}")
 
         client = tornado.httpclient.AsyncHTTPClient()
         try:
             res = await client.fetch(self.subreq)
         except ConnectionRefusedError:
             self.logger.info(
-                "couldn't register with datafeed router "
-                "on {0!s}: connection refused".format(self.subreq.url))
+                "couldn't register with "
+                f"datafeed router  {self.subreq.url!s}: connection refused")
             return
         except tornado.httpclient.HTTPError as err:
             self.logger.info(
                 "couldn't register with "
-                "datafeed router on {0!s}: {1!s}".format(self.subreq.url, err))
+                f"datafeed router {self.subreq.url!s}: {err!s}")
             return
 
         if res.code == 201:
